use std::sync::Arc;
use sqlx::PgPool;
use crate::model::database::User;

#[derive(Debug)]
pub(crate) enum UserNotFoundError {
    Uuid(uuid::Error),
    Sqlx(sqlx::Error),
    NotFound(String),
}

impl From<uuid::Error> for UserNotFoundError {
    fn from(e: uuid::Error) -> Self { UserNotFoundError::Uuid(e)}
}

impl From<sqlx::Error> for UserNotFoundError {
    fn from(e: sqlx::Error) -> Self { UserNotFoundError::Sqlx(e) }
}

pub(crate) async fn get_user_by_token(
    token: String,
    db: Arc<PgPool>
) -> Result<User, UserNotFoundError> {
    sqlx::query_as!(
        User,
        "SELECT u.* FROM \"user\" u JOIN webhook w ON w.user = u.uuid WHERE w.token = $1",
        token
    ).fetch_one(db.as_ref())
        .await
        .map_err(|err| err.into())
}