use std::collections::HashMap;
use std::sync::Arc;
use sqlx::{PgPool};
use sqlx::pool::PoolOptions;
use tokio::sync::Mutex;
use crate::config::IceBingoConfig;

#[derive(Clone)]
pub(crate) struct AppState
{
    pub state: Arc<Mutex<HashMap<String, String>>>,
    pub db: Arc<PgPool>,
}

impl AppState {
    pub(crate) async fn new(config: IceBingoConfig) -> Result<Self, sqlx::Error> {
        let kv_store: HashMap<String, String> = HashMap::new();
        let state_mutex = Arc::new(Mutex::new(kv_store));
        let db_url = get_db_url(&config);
        let pool_options = PoolOptions::new()
            .max_connections(20)
            .min_connections(5);
        let pool = pool_options.connect(db_url.as_str()).await?;
        Ok(Self {
            state: state_mutex,
            db: Arc::new(pool)
        })
    }
}

fn get_db_url(config: &IceBingoConfig) -> String {
    match config.database.db_type.as_str() {
        "postgresql" => format!(
            "postgresql://{}:{}@{}/{}",
            config.database.user,
            config.database.password,
            config.database.host,
            config.database.database
        ),
        _ => "".to_string()
    }
}

