use serde::{de, Deserialize};
use strum_macros::{Display, EnumString};

#[derive(Debug, Deserialize)]
pub(crate) struct CheckIn {
    #[serde(deserialize_with = "reason_to_enum")]
    pub reason: CheckInReason,
    pub status: CheckInStatus,
}

#[derive(Debug, Deserialize, EnumString)]
#[strum(ascii_case_insensitive)]
pub enum CheckInReason {
    PING,
    CHECKIN,
    UPDATE,
    UNDO,
}

fn reason_to_enum<'de, D>(deserializer: D) -> Result<CheckInReason, D::Error>
where
    D: de::Deserializer<'de>,
{
    let s: &str = de::Deserialize::deserialize(deserializer)?;
    CheckInReason::try_from(s).map_err(de::Error::custom)
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct CheckInStatus {
    action_time: usize,
    backend: Backend,
    checked_in: bool,
    comment: Option<String>,
    deprecated: bool,
    pub from_station: Station,
    //pub to_station: Station,
    pub train: Train
}

#[derive(Debug, Deserialize, EnumString)]
#[strum(ascii_case_insensitive)]
enum BackendType {
    DBRIS,
}

#[derive(Debug, Deserialize)]
struct Backend {
    id: u8,
    name: String,
    #[serde(rename = "type")]
    backend_type: BackendType,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Station {
    ds100: Option<String>,
    latitude: Option<f64>,
    longitude: Option<f64>,
    platform: Option<String>,
    real_time: Option<usize>,
    pub scheduled_time: Option<usize>,
    pub uic: usize,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Train {
    hafas_id: Option<String>,
    id: String,
    line: Option<String>,
    #[serde(rename = "no")]
    pub number: Option<String>,
    #[serde(rename = "type")]
    pub train_type: Option<TrainType>,
}

#[derive(Debug, Display, Deserialize, EnumString, Copy, Clone)]
#[serde(rename_all = "UPPERCASE")]
pub enum TrainType {
    ICE,
    IC,
    ECE,
    EC,
    RE,
    RB,
    MEX,
    S,
    U,
    BUS,
}
