use axum::http::request::Parts;
use axum::http::StatusCode;
use axum::extract::FromRequestParts;
use axum_auth::{AuthBearer, Rejection};
use axum_core::extract::FromRef;
use crate::database::{get_user_by_token, UserNotFoundError};
use crate::model::app::AppState;
use crate::model::database::User;

pub(crate) struct UserBearerTokenExtractor(pub User);
pub(crate) type InvalidBearerTokenRejection = Rejection;

impl From<UserNotFoundError> for InvalidBearerTokenRejection {
    fn from(e: UserNotFoundError) -> Self {
        (
            StatusCode::UNAUTHORIZED,
            "No user associated with this bearer token"
        ).into()
    }
}

impl<P> FromRequestParts<P> for UserBearerTokenExtractor
where
    AppState: FromRef<P>,
    P: Send + Sync,
{
    type Rejection = InvalidBearerTokenRejection;

    async fn from_request_parts(parts: &mut Parts, state: &P) -> Result<Self, Self::Rejection>
    {
        let AuthBearer(bearer_token) = AuthBearer::from_request_parts(parts, state)
            .await?;
        let state = AppState::from_ref(state);
        let user = get_user_by_token(bearer_token, state.db).await?;
        Ok(UserBearerTokenExtractor(user))
    }
}
